% Compute average accuracy of label prediction on held-out samples of Cho et al.
% data. 
%
% A number of models for joint clustering of gene expression levels and GO tags
% are learnt using EM with different random initializations. Accuracy of label
% prediction is computed for each model and the accuracies are averaged. Average
% accuracy of label prediction by clustering only gene expression levels is also
% computed.

function [] = test_predict_max_lhood_label_tied_params_cho(n_tries, ...
                                                           acc_results_file, ...
                                                           go_dag, ...
                                                           n_closest_genes, ...
                                                           n_clusters, ...
                                                           stage_labels_file)

addpath(genpath('..'));

n_em_iters                    = 40;
do_hard_cluster               = 1;
do_weight_GO_tags             = 0;
do_marginalize                = 1;
ignore_GO_tags_for_pred       = 0;
do_state_pred_from_time       = 0;
forced_state_switch_pts_cy_1  = [4 6 8];
forced_state_switch_pts_cy_2  = [3 5 7];                
do_weighted_expr_avg_for_pred = 0;
do_direct_inference           = 0;
do_gene_identifiability       = 0;

n_states = 4;

results_dir = 'models/cho_dataset/em_clustering_with_GO_tags_tied_params/pred-w-GO-tags-wo-weights';

train_files = {'data/cho_dataset/gene_expressions_421_shift_log_cy_1_geo_norm', ...
               'data/cho_dataset/gene_expressions_421_shift_log_cy_1_geo_norm', ...
               'data/cho_dataset/gene_expressions_421_shift_log_cy_2_geo_norm', ...
               'data/cho_dataset/gene_expressions_421_shift_log_cy_2_geo_norm'};

test_files = {'data/cho_dataset/gene_expressions_421_shift_log_cy_1_geo_norm', ...
              'data/cho_dataset/gene_expressions_421_shift_log_cy_2_geo_norm', ...
              'data/cho_dataset/gene_expressions_421_shift_log_cy_2_geo_norm', ...
              'data/cho_dataset/gene_expressions_421_shift_log_cy_1_geo_norm'};

ng_go_tags_files = {'data/cho_dataset/no_go_tags_421', ...
                    'data/cho_dataset/no_go_tags_421', ...
                    'data/cho_dataset/no_go_tags_421', ...
                    'data/cho_dataset/no_go_tags_421'};

bp_go_tags_files = {'data/cho_dataset/go_tags_bp_w_anc_421', ...
                    'data/cho_dataset/go_tags_bp_w_anc_421', ...
                    'data/cho_dataset/go_tags_bp_w_anc_421', ...
                    'data/cho_dataset/go_tags_bp_w_anc_421'};
                    
mf_go_tags_files = {'data/cho_dataset/go_tags_mf_w_anc_421', ...
                    'data/cho_dataset/go_tags_mf_w_anc_421', ...
                    'data/cho_dataset/go_tags_mf_w_anc_421', ...
                    'data/cho_dataset/go_tags_mf_w_anc_421'};
                    
ng_clusters_files = {strcat(results_dir, '/', 'bayesian_go_input_ng_421_cy_1'), ...
                     strcat(results_dir, '/', 'bayesian_go_input_ng_421_cy_1'), ...
                     strcat(results_dir, '/', 'bayesian_go_input_ng_421_cy_2'), ...
                     strcat(results_dir, '/', 'bayesian_go_input_ng_421_cy_2')};

bp_clusters_files = {strcat(results_dir, '/', 'bayesian_go_input_bp_421_cy_1'), ...
                     strcat(results_dir, '/', 'bayesian_go_input_bp_421_cy_1'), ...
                     strcat(results_dir, '/', 'bayesian_go_input_bp_421_cy_2'), ...
                     strcat(results_dir, '/', 'bayesian_go_input_bp_421_cy_2')};
                 
mf_clusters_files = {strcat(results_dir, '/', 'bayesian_go_input_mf_421_cy_1'), ...
                     strcat(results_dir, '/', 'bayesian_go_input_mf_421_cy_1'), ...
                     strcat(results_dir, '/', 'bayesian_go_input_mf_421_cy_2'), ...
                     strcat(results_dir, '/', 'bayesian_go_input_mf_421_cy_2')};

ng_p_o_bar_c_files ={strcat(results_dir, '/', 'p_o_bar_c_ng_421_cy_1'), ...
                     strcat(results_dir, '/', 'p_o_bar_c_ng_421_cy_1'), ...
                     strcat(results_dir, '/', 'p_o_bar_c_ng_421_cy_2'), ...
                     strcat(results_dir, '/', 'p_o_bar_c_ng_421_cy_2')};

bp_p_o_bar_c_files ={strcat(results_dir, '/', 'p_o_bar_c_bp_w_anc_421_cy_1'), ...
                     strcat(results_dir, '/', 'p_o_bar_c_bp_w_anc_421_cy_1'), ...
                     strcat(results_dir, '/', 'p_o_bar_c_bp_w_anc_421_cy_2'), ...
                     strcat(results_dir, '/', 'p_o_bar_c_bp_w_anc_421_cy_2')};

mf_p_o_bar_c_files ={strcat(results_dir, '/', 'p_o_bar_c_mf_w_anc_421_cy_1'), ...
                     strcat(results_dir, '/', 'p_o_bar_c_mf_w_anc_421_cy_1'), ...
                     strcat(results_dir, '/', 'p_o_bar_c_mf_w_anc_421_cy_2'), ...
                     strcat(results_dir, '/', 'p_o_bar_c_mf_w_anc_421_cy_2')};

ng_model_files = {strcat(results_dir, '/', 'model_ng_421_cy_1.mat'), ... 
                  strcat(results_dir, '/', 'model_ng_421_cy_1.mat'), ...
                  strcat(results_dir, '/', 'model_ng_421_cy_2.mat'), ...
                  strcat(results_dir, '/', 'model_ng_421_cy_2.mat')};

bp_model_files = {strcat(results_dir, '/', 'model_bp_w_anc_421_cy_1.mat'), ... 
                  strcat(results_dir, '/', 'model_bp_w_anc_421_cy_1.mat'), ...
                  strcat(results_dir, '/', 'model_bp_w_anc_421_cy_2.mat'), ...
                  strcat(results_dir, '/', 'model_bp_w_anc_421_cy_2.mat')};

mf_model_files = {strcat(results_dir, '/', 'model_mf_w_anc_421_cy_1.mat'), ... 
                  strcat(results_dir, '/', 'model_mf_w_anc_421_cy_1.mat'), ...
                  strcat(results_dir, '/', 'model_mf_w_anc_421_cy_2.mat'), ...
                  strcat(results_dir, '/', 'model_mf_w_anc_421_cy_2.mat')};

n_exps = length(train_files);

acc_s_ng = zeros(n_exps*n_tries, 1);
acc_s_bp = zeros(n_exps*n_tries, 1);
acc_s_mf = zeros(n_exps*n_tries, 1);

% Making script restartable.
if (exist(acc_results_file , 'file'))
    
    acc_from_file  = load(acc_results_file);
    sum_pred_probs = sum(acc_from_file(:));    

    if (sum_pred_probs > 0)
        acc_s_ng = acc_from_file(:, 1);
        acc_s_bp = acc_from_file(:, 2);
        acc_s_mf = acc_from_file(:, 3);
    end

else
    
    acc_from_file = [acc_s_ng acc_s_bp acc_s_mf];

end

% Writing stage prediction labels.
max_n_samples     = 9;
stage_pred_labels = zeros(n_exps*n_tries, 3*max_n_samples);

for i_exp = 1:n_exps
    train_file = train_files{i_exp};
    test_file  = test_files{i_exp};

    if ( strcmp(go_dag, 'ng') )

        % No GO tags.
        go_tags_file   = ng_go_tags_files{i_exp};
        clusters_file  = ng_clusters_files{i_exp};
        p_o_bar_c_file = ng_p_o_bar_c_files{i_exp};
        model_mat_file = ng_model_files{i_exp};

        for trial = 1:n_tries
            
            sum_pred_probs = sum(acc_from_file((((i_exp-1)*n_tries)+trial): ...
                                                (n_exps*n_tries), 1));
            
            if ( sum_pred_probs  > 0 )
            
                exp_str   = num2str(i_exp);
                trial_str = num2str(trial);

                disp(strcat('Skipping trial: ', trial_str, ' for experiment: ', exp_str));
                continue;

            else

                if (~isempty(strfind(train_file, 'cy_1')))
                    forced_state_switch_pts = forced_state_switch_pts_cy_1;
                else
                    forced_state_switch_pts = forced_state_switch_pts_cy_2;
                end
                
                cluster_with_em_GO_tags_tied_params(train_file, ...
						    go_tags_file, ...
						    n_states, ...
						    n_clusters, ...
						    n_em_iters, ...
						    do_hard_cluster, ...
						    do_weight_GO_tags, ...
						    clusters_file, ...
						    p_o_bar_c_file, ...
						    model_mat_file, ...
						    forced_state_switch_pts);

                [time_pred_probs, state_pred_probs] = predict_max_lhood_label_tied_params(train_file, ...
										          test_file, ...
											  go_tags_file, ...
											  model_mat_file, ...
											  do_marginalize, ...
											  do_weight_GO_tags, ...
											  ignore_GO_tags_for_pred, ...
											  n_closest_genes, ...
											  do_weighted_expr_avg_for_pred, ...
											  do_direct_inference, ...
											  do_gene_identifiability, ...
											  forced_state_switch_pts);

                n_samples   = size(time_pred_probs, 1);
                n_correct_s = 0;

                for s = 1:n_samples
                    if (do_state_pred_from_time)
                        if (~isempty(strfind(train_file, 'cy_2')))

                            state_pred_probs(s, :) = [sum(time_pred_probs(s, 1:(forced_state_switch_pts_cy_2(1)-1))), ...
                                                      sum(time_pred_probs(s, forced_state_switch_pts_cy_2(1):(forced_state_switch_pts_cy_2(2)-1))), ...
                                                      sum(time_pred_probs(s, forced_state_switch_pts_cy_2(2):(forced_state_switch_pts_cy_2(3)-1))), ...
                                                      sum(time_pred_probs(s, forced_state_switch_pts_cy_2(3):8))];

                        else

                            state_pred_probs(s, :) = [sum(time_pred_probs(s, 1:(forced_state_switch_pts_cy_1(1)-1))), ...
                                                      sum(time_pred_probs(s, forced_state_switch_pts_cy_1(1):(forced_state_switch_pts_cy_1(2)-1))), ...
                                                      sum(time_pred_probs(s, forced_state_switch_pts_cy_1(2):(forced_state_switch_pts_cy_1(3)-1))), ...
                                                      sum(time_pred_probs(s, forced_state_switch_pts_cy_1(3):9))];
                            
                        end
                    end
                    
                    [max_prob, pred_state] = max(state_pred_probs(s, :));

                    stage_pred_labels(((i_exp-1)*n_tries)+trial, s) = pred_state;

                    if (~isempty(strfind(test_file, 'cy_1')))

                        if ( (s >= 1) & (s < forced_state_switch_pts_cy_1(1)) )                
                            if (pred_state == 1)
                                n_correct_s = n_correct_s + 1;
                            end
                        elseif ( (s >= forced_state_switch_pts_cy_1(1)) & (s < forced_state_switch_pts_cy_1(2)) )
                            if (pred_state == 2)
                                n_correct_s = n_correct_s + 1;
                            end
                        elseif ( (s >= forced_state_switch_pts_cy_1(2)) & (s < forced_state_switch_pts_cy_1(3)) )
                            if (pred_state == 3)
                                n_correct_s = n_correct_s + 1;
                            end
                        else %( (s >= forced_state_switch_pts_cy_1(3)) & (s <= 9) )
                            if (pred_state == 4)
                                n_correct_s = n_correct_s + 1;
                            end
                        end

                    else

                        if ( (s >= 1) & (s < forced_state_switch_pts_cy_2(1)) )                
                            if (pred_state == 1)
                                n_correct_s = n_correct_s + 1;
                            end
                        elseif ( (s >= forced_state_switch_pts_cy_2(1)) & (s < forced_state_switch_pts_cy_2(2)) )
                            if (pred_state == 2)
                                n_correct_s = n_correct_s + 1;
                            end
                        elseif ( (s >= forced_state_switch_pts_cy_2(2)) & (s < forced_state_switch_pts_cy_2(3)) )
                            if (pred_state == 3)
                                n_correct_s = n_correct_s + 1;
                            end
                        else %( (s >= forced_state_switch_pts_cy_2(3)) & (s <= 8) )
                            if (pred_state == 4)
                                n_correct_s = n_correct_s + 1;
                            end
                        end
                        
                    end
                end
                acc_s_ng(((i_exp-1)*n_tries)+trial) = n_correct_s / n_samples;        
                
                %Update results.
                acc_results = [acc_s_ng acc_s_bp acc_s_mf];    
                save(acc_results_file, 'acc_results', '-ASCII');

                save(stage_labels_file, 'stage_pred_labels', '-ASCII');
            end

        end

    end

    if ( strcmp(go_dag, 'bp') )

        % BP GO tags.
        go_tags_file   = bp_go_tags_files{i_exp};
        clusters_file  = bp_clusters_files{i_exp};
        p_o_bar_c_file = bp_p_o_bar_c_files{i_exp};
        model_mat_file = bp_model_files{i_exp};

        for trial = 1:n_tries
            
            sum_pred_probs = sum(acc_from_file((((i_exp-1)*n_tries)+trial): ...
                                                (n_exps*n_tries), 2));
            
            if ( sum_pred_probs  > 0 )
            
                exp_str   = num2str(i_exp);
                trial_str = num2str(trial);

                disp(strcat('Skipping trial: ', trial_str, ' for experiment: ', exp_str));
                continue;

            else

                if (~isempty(strfind(train_file, 'cy_1')))
                    forced_state_switch_pts = forced_state_switch_pts_cy_1;
                else
                    forced_state_switch_pts = forced_state_switch_pts_cy_2;
                end

                cluster_with_em_GO_tags_tied_params(train_file, ...
						    go_tags_file, ...
						    n_states, ...
						    n_clusters, ...
						    n_em_iters, ...
						    do_hard_cluster, ...
						    do_weight_GO_tags, ...
						    clusters_file, ...
						    p_o_bar_c_file, ...
						    model_mat_file, ...
						    forced_state_switch_pts);

                [time_pred_probs, state_pred_probs] = predict_max_lhood_label_tied_params(train_file, ...
										          test_file, ...
											  go_tags_file, ...
											  model_mat_file, ...
											  do_marginalize, ...
											  do_weight_GO_tags, ...
											  ignore_GO_tags_for_pred, ...
											  n_closest_genes, ...
											  do_weighted_expr_avg_for_pred, ...
											  do_direct_inference, ...
											  do_gene_identifiability, ...
											  forced_state_switch_pts);

                n_samples   = size(time_pred_probs, 1);
                n_correct_s = 0;

                for s = 1:n_samples
                    if (do_state_pred_from_time)
                        if (~isempty(strfind(train_file, 'cy_2')))

                            state_pred_probs(s, :) = [sum(time_pred_probs(s, 1:(forced_state_switch_pts_cy_2(1)-1))), ...
                                                      sum(time_pred_probs(s, forced_state_switch_pts_cy_2(1):(forced_state_switch_pts_cy_2(2)-1))), ...
                                                      sum(time_pred_probs(s, forced_state_switch_pts_cy_2(2):(forced_state_switch_pts_cy_2(3)-1))), ...
                                                      sum(time_pred_probs(s, forced_state_switch_pts_cy_2(3):8))];

                        else

                            state_pred_probs(s, :) = [sum(time_pred_probs(s, 1:(forced_state_switch_pts_cy_1(1)-1))), ...
                                                      sum(time_pred_probs(s, forced_state_switch_pts_cy_1(1):(forced_state_switch_pts_cy_1(2)-1))), ...
                                                      sum(time_pred_probs(s, forced_state_switch_pts_cy_1(2):(forced_state_switch_pts_cy_1(3)-1))), ...
                                                      sum(time_pred_probs(s, forced_state_switch_pts_cy_1(3):9))];
                            
                        end
                    end
                    
                    [max_prob, pred_state] = max(state_pred_probs(s, :));

                    stage_pred_labels(((i_exp-1)*n_tries)+trial, max_n_samples+s) = pred_state;

                    if (~isempty(strfind(test_file, 'cy_1')))

                        if ( (s >= 1) & (s < forced_state_switch_pts_cy_1(1)) )                
                            if (pred_state == 1)
                                n_correct_s = n_correct_s + 1;
                            end
                        elseif ( (s >= forced_state_switch_pts_cy_1(1)) & (s < forced_state_switch_pts_cy_1(2)) )
                            if (pred_state == 2)
                                n_correct_s = n_correct_s + 1;
                            end
                        elseif ( (s >= forced_state_switch_pts_cy_1(2)) & (s < forced_state_switch_pts_cy_1(3)) )
                            if (pred_state == 3)
                                n_correct_s = n_correct_s + 1;
                            end
                        else %( (s >= forced_state_switch_pts_cy_1(3)) & (s <= 9) )
                            if (pred_state == 4)
                                n_correct_s = n_correct_s + 1;
                            end
                        end

                    else

                        if ( (s >= 1) & (s < forced_state_switch_pts_cy_2(1)) )                
                            if (pred_state == 1)
                                n_correct_s = n_correct_s + 1;
                            end
                        elseif ( (s >= forced_state_switch_pts_cy_2(1)) & (s < forced_state_switch_pts_cy_2(2)) )
                            if (pred_state == 2)
                                n_correct_s = n_correct_s + 1;
                            end
                        elseif ( (s >= forced_state_switch_pts_cy_2(2)) & (s < forced_state_switch_pts_cy_2(3)) )
                            if (pred_state == 3)
                                n_correct_s = n_correct_s + 1;
                            end
                        else %( (s >= forced_state_switch_pts_cy_2(3)) & (s <= 8) )
                            if (pred_state == 4)
                                n_correct_s = n_correct_s + 1;
                            end
                        end
                        
                    end
                end
                acc_s_bp(((i_exp-1)*n_tries)+trial) = n_correct_s / n_samples;

                %Update results.
                acc_results = [acc_s_ng acc_s_bp acc_s_mf];    
                save(acc_results_file, 'acc_results', '-ASCII');

                save(stage_labels_file, 'stage_pred_labels', '-ASCII');
            end

        end

    end

    if ( strcmp(go_dag, 'mf') )

        % MF GO tags.
        go_tags_file   = mf_go_tags_files{i_exp};
        clusters_file  = mf_clusters_files{i_exp};
        p_o_bar_c_file = mf_p_o_bar_c_files{i_exp};
        model_mat_file = mf_model_files{i_exp};

        for trial = 1:n_tries
            
            sum_pred_probs = sum(acc_from_file((((i_exp-1)*n_tries)+trial): ...
                                                (n_exps*n_tries), 3));
            
            if ( sum_pred_probs  > 0 )
            
                exp_str   = num2str(i_exp);
                trial_str = num2str(trial);

                disp(strcat('Skipping trial: ', trial_str, ' for experiment: ', exp_str));
                continue;

            else

                if (~isempty(strfind(train_file, 'cy_1')))
                    forced_state_switch_pts = forced_state_switch_pts_cy_1;
                else
                    forced_state_switch_pts = forced_state_switch_pts_cy_2;
                end

                cluster_with_em_GO_tags_tied_params(train_file, ...
						    go_tags_file, ...
						    n_states, ...
						    n_clusters, ...
						    n_em_iters, ...
						    do_hard_cluster, ...
						    do_weight_GO_tags, ...
						    clusters_file, ...
						    p_o_bar_c_file, ...
						    model_mat_file, ...
						    forced_state_switch_pts);

                [time_pred_probs, state_pred_probs] = predict_max_lhood_label_tied_params(train_file, ...
										          test_file, ...
											  go_tags_file, ...
											  model_mat_file, ...
											  do_marginalize, ...
											  do_weight_GO_tags, ...
											  ignore_GO_tags_for_pred, ...
											  n_closest_genes, ...
											  do_weighted_expr_avg_for_pred, ...
											  do_direct_inference, ...
											  do_gene_identifiability, ...
											  forced_state_switch_pts);

                n_samples   = size(time_pred_probs, 1);
                n_correct_s = 0;

                for s = 1:n_samples
                    if (do_state_pred_from_time)
                        if (~isempty(strfind(train_file, 'cy_2')))

                            state_pred_probs(s, :) = [sum(time_pred_probs(s, 1:(forced_state_switch_pts_cy_2(1)-1))), ...
                                                      sum(time_pred_probs(s, forced_state_switch_pts_cy_2(1):(forced_state_switch_pts_cy_2(2)-1))), ...
                                                      sum(time_pred_probs(s, forced_state_switch_pts_cy_2(2):(forced_state_switch_pts_cy_2(3)-1))), ...
                                                      sum(time_pred_probs(s, forced_state_switch_pts_cy_2(3):8))];

                        else

                            state_pred_probs(s, :) = [sum(time_pred_probs(s, 1:(forced_state_switch_pts_cy_1(1)-1))), ...
                                                      sum(time_pred_probs(s, forced_state_switch_pts_cy_1(1):(forced_state_switch_pts_cy_1(2)-1))), ...
                                                      sum(time_pred_probs(s, forced_state_switch_pts_cy_1(2):(forced_state_switch_pts_cy_1(3)-1))), ...
                                                      sum(time_pred_probs(s, forced_state_switch_pts_cy_1(3):9))];
                            
                        end
                    end
                    
                    [max_prob, pred_state] = max(state_pred_probs(s, :));

                    stage_pred_labels(((i_exp-1)*n_tries)+trial, (2*max_n_samples)+s) = pred_state;

                    if (~isempty(strfind(test_file, 'cy_1')))

                        if ( (s >= 1) & (s < forced_state_switch_pts_cy_1(1)) )                
                            if (pred_state == 1)
                                n_correct_s = n_correct_s + 1;
                            end
                        elseif ( (s >= forced_state_switch_pts_cy_1(1)) & (s < forced_state_switch_pts_cy_1(2)) )
                            if (pred_state == 2)
                                n_correct_s = n_correct_s + 1;
                            end
                        elseif ( (s >= forced_state_switch_pts_cy_1(2)) & (s < forced_state_switch_pts_cy_1(3)) )
                            if (pred_state == 3)
                                n_correct_s = n_correct_s + 1;
                            end
                        else %( (s >= forced_state_switch_pts_cy_1(3)) & (s <= 9) )
                            if (pred_state == 4)
                                n_correct_s = n_correct_s + 1;
                            end
                        end

                    else

                        if ( (s >= 1) & (s < forced_state_switch_pts_cy_2(1)) )                
                            if (pred_state == 1)
                                n_correct_s = n_correct_s + 1;
                            end
                        elseif ( (s >= forced_state_switch_pts_cy_2(1)) & (s < forced_state_switch_pts_cy_2(2)) )
                            if (pred_state == 2)
                                n_correct_s = n_correct_s + 1;
                            end
                        elseif ( (s >= forced_state_switch_pts_cy_2(2)) & (s < forced_state_switch_pts_cy_2(3)) )
                            if (pred_state == 3)
                                n_correct_s = n_correct_s + 1;
                            end
                        else %( (s >= forced_state_switch_pts_cy_2(3)) & (s <= 8) )
                            if (pred_state == 4)
                                n_correct_s = n_correct_s + 1;
                            end
                        end
                        
                    end
                end
                acc_s_mf(((i_exp-1)*n_tries)+trial) = n_correct_s / n_samples;
                
                %Update results.
                acc_results = [acc_s_ng acc_s_bp acc_s_mf];    
                save(acc_results_file, 'acc_results', '-ASCII');

                save(stage_labels_file, 'stage_pred_labels', '-ASCII');
            end

        end

    end

end
