% Compute average accuracy of label prediction on held-out samples of Hoying
% data. 
%
% A number of models for joint clustering of gene expression levels and GO tags
% are learnt using EM with different random initializations. Accuracy of label
% prediction is computed for each model and the accuracies are averaged. Average
% accuracy of label prediction by clustering only gene expression levels is also
% computed.

function [] = test_predict_max_lhood_label_tied_params_hoying(n_tries, ...
                                                              acc_results_file, ...
                                                              go_dag, ...
                                                              n_closest_genes, ...
                                                              n_clusters, ...
                                                              time_labels_file, ...
                                                              stage_labels_file)

addpath(genpath('..'));

n_em_iters                    = 40;
do_hard_cluster               = 1;
do_weight_GO_tags             = 0;
do_marginalize                = 1;
ignore_GO_tags_for_pred       = 0;
stage_switch_time_point       = 3;
forced_state_switch_pts       = [stage_switch_time_point];
do_weighted_expr_avg_for_pred = 0;
do_direct_inference           = 0;
do_gene_identifiability       = 0;

n_states = 2;

exp_one_results_dir = 'models/hoying_dataset/em_clustering_with_GO_tags_tied_params/exp_one/pred-w-GO-tags-wo-weights';
exp_two_results_dir = 'models/hoying_dataset/em_clustering_with_GO_tags_tied_params/exp_two/pred-w-GO-tags-wo-weights';

train_files = {'data/hoying_dataset/exp_one/gene_expressions_1282_exp_one_subset_data_mp', ...
               'data/hoying_dataset/exp_one/gene_expressions_1282_exp_one_subset_data_mp', ...
               'data/hoying_dataset/exp_two/gene_expressions_1282_exp_two_subset_data_mp', ...
               'data/hoying_dataset/exp_two/gene_expressions_1282_exp_two_subset_data_mp', ...
               'data/hoying_dataset/exp_two/gene_expressions_978_exp_two_subset_data_mp', ...
               'data/hoying_dataset/exp_two/gene_expressions_978_exp_two_subset_data_mp', ...
               'data/hoying_dataset/exp_one/gene_expressions_978_exp_one_subset_data_mp', ...
               'data/hoying_dataset/exp_one/gene_expressions_978_exp_one_subset_data_mp'};

test_files = {'data/hoying_dataset/exp_one/gene_expressions_1282_exp_one_subset_data_mp', ...
              'data/hoying_dataset/exp_two/gene_expressions_1282_exp_two_subset_data_mp', ...
              'data/hoying_dataset/exp_two/gene_expressions_1282_exp_two_subset_data_mp', ...
              'data/hoying_dataset/exp_one/gene_expressions_1282_exp_one_subset_data_mp', ...
              'data/hoying_dataset/exp_two/gene_expressions_978_exp_two_subset_data_mp', ...
              'data/hoying_dataset/exp_one/gene_expressions_978_exp_one_subset_data_mp', ...
              'data/hoying_dataset/exp_one/gene_expressions_978_exp_one_subset_data_mp', ...
              'data/hoying_dataset/exp_two/gene_expressions_978_exp_two_subset_data_mp'};

ng_go_tags_files = {'data/hoying_dataset/exp_one/no_go_tags_1282_subset', ...
                    'data/hoying_dataset/exp_one/no_go_tags_1282_subset', ...
                    'data/hoying_dataset/exp_one/no_go_tags_1282_subset', ...
                    'data/hoying_dataset/exp_one/no_go_tags_1282_subset', ...
                    'data/hoying_dataset/exp_two/no_go_tags_978_subset', ...
                    'data/hoying_dataset/exp_two/no_go_tags_978_subset', ...
                    'data/hoying_dataset/exp_two/no_go_tags_978_subset', ...
                    'data/hoying_dataset/exp_two/no_go_tags_978_subset'};
                    
bp_go_tags_files = {'data/hoying_dataset/exp_one/go_tags_bp_w_anc_1282_subset', ...
                    'data/hoying_dataset/exp_one/go_tags_bp_w_anc_1282_subset', ...
                    'data/hoying_dataset/exp_one/go_tags_bp_w_anc_1282_subset', ...
                    'data/hoying_dataset/exp_one/go_tags_bp_w_anc_1282_subset', ...
                    'data/hoying_dataset/exp_two/go_tags_bp_w_anc_978_subset', ...
                    'data/hoying_dataset/exp_two/go_tags_bp_w_anc_978_subset', ...
                    'data/hoying_dataset/exp_two/go_tags_bp_w_anc_978_subset', ...
                    'data/hoying_dataset/exp_two/go_tags_bp_w_anc_978_subset'};                   

mf_go_tags_files = {'data/hoying_dataset/exp_one/go_tags_mf_w_anc_1282_subset', ...
                    'data/hoying_dataset/exp_one/go_tags_mf_w_anc_1282_subset', ...
                    'data/hoying_dataset/exp_one/go_tags_mf_w_anc_1282_subset', ...
                    'data/hoying_dataset/exp_one/go_tags_mf_w_anc_1282_subset', ...
                    'data/hoying_dataset/exp_two/go_tags_mf_w_anc_978_subset', ...
                    'data/hoying_dataset/exp_two/go_tags_mf_w_anc_978_subset', ...
                    'data/hoying_dataset/exp_two/go_tags_mf_w_anc_978_subset', ...
                    'data/hoying_dataset/exp_two/go_tags_mf_w_anc_978_subset'};                

ng_clusters_files = {strcat(exp_one_results_dir, '/', 'bayesian_go_input_ng_1282_subset.txt'), ...
                     strcat(exp_one_results_dir, '/', 'bayesian_go_input_ng_1282_subset.txt'), ...
                     strcat(exp_two_results_dir, '/', 'bayesian_go_input_ng_1282_subset.txt'), ...
                     strcat(exp_two_results_dir, '/', 'bayesian_go_input_ng_1282_subset.txt'), ...
                     strcat(exp_two_results_dir, '/', 'bayesian_go_input_ng_978_subset.txt'), ...
                     strcat(exp_two_results_dir, '/', 'bayesian_go_input_ng_978_subset.txt'), ...
                     strcat(exp_one_results_dir, '/', 'bayesian_go_input_ng_978_subset.txt'), ...
                     strcat(exp_one_results_dir, '/', 'bayesian_go_input_ng_978_subset.txt')};

bp_clusters_files = {strcat(exp_one_results_dir, '/', 'bayesian_go_input_bp_1282_subset.txt'), ...
                     strcat(exp_one_results_dir, '/', 'bayesian_go_input_bp_1282_subset.txt'), ...
                     strcat(exp_two_results_dir, '/', 'bayesian_go_input_bp_1282_subset.txt'), ...
                     strcat(exp_two_results_dir, '/', 'bayesian_go_input_bp_1282_subset.txt'), ...
                     strcat(exp_two_results_dir, '/', 'bayesian_go_input_bp_978_subset.txt'), ...
                     strcat(exp_two_results_dir, '/', 'bayesian_go_input_bp_978_subset.txt'), ...
                     strcat(exp_one_results_dir, '/', 'bayesian_go_input_bp_978_subset.txt'), ...
                     strcat(exp_one_results_dir, '/', 'bayesian_go_input_bp_978_subset.txt')};

mf_clusters_files = {strcat(exp_one_results_dir, '/', 'bayesian_go_input_mf_1282_subset.txt'), ...
                     strcat(exp_one_results_dir, '/', 'bayesian_go_input_mf_1282_subset.txt'), ...
                     strcat(exp_two_results_dir, '/', 'bayesian_go_input_mf_1282_subset.txt'), ...
                     strcat(exp_two_results_dir, '/', 'bayesian_go_input_mf_1282_subset.txt'), ...
                     strcat(exp_two_results_dir, '/', 'bayesian_go_input_mf_978_subset.txt'), ...
                     strcat(exp_two_results_dir, '/', 'bayesian_go_input_mf_978_subset.txt'), ...
                     strcat(exp_one_results_dir, '/', 'bayesian_go_input_mf_978_subset.txt'), ...
                     strcat(exp_one_results_dir, '/', 'bayesian_go_input_mf_978_subset.txt')};

ng_p_o_bar_c_files = {strcat(exp_one_results_dir, '/', 'p_o_bar_c_ng_1282_subset.txt'), ...
                      strcat(exp_one_results_dir, '/', 'p_o_bar_c_ng_1282_subset.txt'), ...
                      strcat(exp_two_results_dir, '/', 'p_o_bar_c_ng_1282_subset.txt'), ...
                      strcat(exp_two_results_dir, '/', 'p_o_bar_c_ng_1282_subset.txt'), ...
                      strcat(exp_two_results_dir, '/', 'p_o_bar_c_ng_978_subset.txt'), ...
                      strcat(exp_two_results_dir, '/', 'p_o_bar_c_ng_978_subset.txt'), ...
                      strcat(exp_one_results_dir, '/', 'p_o_bar_c_ng_978_subset.txt'), ...
                      strcat(exp_one_results_dir, '/', 'p_o_bar_c_ng_978_subset.txt')};

bp_p_o_bar_c_files = {strcat(exp_one_results_dir, '/', 'p_o_bar_c_bp_1282_subset.txt'), ...
                      strcat(exp_one_results_dir, '/', 'p_o_bar_c_bp_1282_subset.txt'), ...
                      strcat(exp_two_results_dir, '/', 'p_o_bar_c_bp_1282_subset.txt'), ...
                      strcat(exp_two_results_dir, '/', 'p_o_bar_c_bp_1282_subset.txt'), ...
                      strcat(exp_two_results_dir, '/', 'p_o_bar_c_bp_978_subset.txt'), ...
                      strcat(exp_two_results_dir, '/', 'p_o_bar_c_bp_978_subset.txt'), ...
                      strcat(exp_one_results_dir, '/', 'p_o_bar_c_bp_978_subset.txt'), ...
                      strcat(exp_one_results_dir, '/', 'p_o_bar_c_bp_978_subset.txt')};

mf_p_o_bar_c_files = {strcat(exp_one_results_dir, '/', 'p_o_bar_c_mf_1282_subset.txt'), ...
                      strcat(exp_one_results_dir, '/', 'p_o_bar_c_mf_1282_subset.txt'), ...
                      strcat(exp_two_results_dir, '/', 'p_o_bar_c_mf_1282_subset.txt'), ...
                      strcat(exp_two_results_dir, '/', 'p_o_bar_c_mf_1282_subset.txt'), ...
                      strcat(exp_two_results_dir, '/', 'p_o_bar_c_mf_978_subset.txt'), ...
                      strcat(exp_two_results_dir, '/', 'p_o_bar_c_mf_978_subset.txt'), ...
                      strcat(exp_one_results_dir, '/', 'p_o_bar_c_mf_978_subset.txt'), ...
                      strcat(exp_one_results_dir, '/', 'p_o_bar_c_mf_978_subset.txt')};

ng_model_files = {strcat(exp_one_results_dir, '/', 'model_ng_1282_subset.mat'), ...
                  strcat(exp_one_results_dir, '/', 'model_ng_1282_subset.mat'), ...
                  strcat(exp_two_results_dir, '/', 'model_ng_1282_subset.mat'), ...
                  strcat(exp_two_results_dir, '/', 'model_ng_1282_subset.mat'), ...
                  strcat(exp_two_results_dir, '/', 'model_ng_978_subset.mat'), ...
                  strcat(exp_two_results_dir, '/', 'model_ng_978_subset.mat'), ...
                  strcat(exp_one_results_dir, '/', 'model_ng_978_subset.mat'), ...
                  strcat(exp_one_results_dir, '/', 'model_ng_978_subset.mat')};

bp_model_files = {strcat(exp_one_results_dir, '/', 'model_bp_1282_subset.mat'), ...
                  strcat(exp_one_results_dir, '/', 'model_bp_1282_subset.mat'), ...
                  strcat(exp_two_results_dir, '/', 'model_bp_1282_subset.mat'), ...
                  strcat(exp_two_results_dir, '/', 'model_bp_1282_subset.mat'), ...
                  strcat(exp_two_results_dir, '/', 'model_bp_978_subset.mat'), ...
                  strcat(exp_two_results_dir, '/', 'model_bp_978_subset.mat'), ...
                  strcat(exp_one_results_dir, '/', 'model_bp_978_subset.mat'), ...
                  strcat(exp_one_results_dir, '/', 'model_bp_978_subset.mat')};

mf_model_files = {strcat(exp_one_results_dir, '/', 'model_mf_1282_subset.mat'), ...
                  strcat(exp_one_results_dir, '/', 'model_mf_1282_subset.mat'), ...
                  strcat(exp_two_results_dir, '/', 'model_mf_1282_subset.mat'), ...
                  strcat(exp_two_results_dir, '/', 'model_mf_1282_subset.mat'), ...
                  strcat(exp_two_results_dir, '/', 'model_mf_978_subset.mat'), ...
                  strcat(exp_two_results_dir, '/', 'model_mf_978_subset.mat'), ...
                  strcat(exp_one_results_dir, '/', 'model_mf_978_subset.mat'), ...
                  strcat(exp_one_results_dir, '/', 'model_mf_978_subset.mat')};

n_exps = length(train_files);

acc_t_ng_eo_et = zeros(n_exps*n_tries, 1);
acc_s_ng_eo_et = zeros(n_exps*n_tries, 1);
acc_t_bp_eo_et = zeros(n_exps*n_tries, 1);
acc_s_bp_eo_et = zeros(n_exps*n_tries, 1);
acc_t_mf_eo_et = zeros(n_exps*n_tries, 1);
acc_s_mf_eo_et = zeros(n_exps*n_tries, 1);

% Making script restartable.
if (exist(acc_results_file , 'file'))
    
    acc_from_file  = load(acc_results_file);
    sum_pred_probs = sum(acc_from_file(:));    

    if (sum_pred_probs > 0)
        acc_t_ng_eo_et = acc_from_file(:, 1);
        acc_s_ng_eo_et = acc_from_file(:, 2);
        acc_t_bp_eo_et = acc_from_file(:, 3);
        acc_s_bp_eo_et = acc_from_file(:, 4);
        acc_t_mf_eo_et = acc_from_file(:, 5);
        acc_s_mf_eo_et = acc_from_file(:, 6);
    end

else
    
    acc_from_file = [acc_t_ng_eo_et acc_s_ng_eo_et acc_t_bp_eo_et acc_s_bp_eo_et acc_t_mf_eo_et acc_s_mf_eo_et];

end

% Writing time and stage prediction labels.
max_n_samples     = 6;
time_pred_labels  = zeros(n_exps*n_tries, 3*max_n_samples);
stage_pred_labels = zeros(n_exps*n_tries, 3*max_n_samples);

for i_exp = 1:n_exps
    %train_file              = 'data/hoying_dataset/exp_one/gene_expressions_1282_exp_one_subset_data_mp';
    %test_file               = 'data/hoying_dataset/exp_two/gene_expressions_1282_exp_two_subset_data_mp';
    train_file = train_files{i_exp}
    test_file  = test_files{i_exp}

    if ( strcmp(go_dag, 'ng') )

        % No GO tags.
        %go_tags_file   = 'data/hoying_dataset/exp_one/no_go_tags_1282_subset';
        %clusters_file  = 'data/hoying_dataset/em_clustering_with_GO_tags/exp_one/bayesian_go_input_ng_1282_subset.txt';
        %p_o_bar_c_file = 'data/hoying_dataset/em_clustering_with_GO_tags/exp_one/p_o_bar_c_ng_1282_subset.txt';
        %model_mat_file = 'data/hoying_dataset/em_clustering_with_GO_tags/exp_one/model_ng_1282_subset.mat';
        go_tags_file   = ng_go_tags_files{i_exp};
        clusters_file  = ng_clusters_files{i_exp};
        p_o_bar_c_file = ng_p_o_bar_c_files{i_exp};
        model_mat_file = ng_model_files{i_exp};

        for trial = 1:n_tries
            
            sum_pred_probs = sum(acc_from_file((((i_exp-1)*n_tries)+trial): ...
                                                (n_exps*n_tries), 1));
            
            if ( sum_pred_probs  > 0 )
            
                exp_str   = num2str(i_exp);
                trial_str = num2str(trial);

                disp(strcat('Skipping trial: ', trial_str, ' for experiment: ', exp_str));
                continue;

            else

                cluster_with_em_GO_tags_tied_params(train_file, ...
						    go_tags_file, ...
						    n_states, ...
						    n_clusters, ...
						    n_em_iters, ...
						    do_hard_cluster, ...
						    do_weight_GO_tags, ...
						    clusters_file, ...
						    p_o_bar_c_file, ...
						    model_mat_file, ...
						    forced_state_switch_pts);

                [time_pred_probs, state_pred_probs] = predict_max_lhood_label_tied_params(train_file, ...
										          test_file, ...
											  go_tags_file, ...
											  model_mat_file, ...
											  do_marginalize, ...
											  do_weight_GO_tags, ...
											  ignore_GO_tags_for_pred, ...
											  n_closest_genes, ...
											  do_weighted_expr_avg_for_pred, ...
											  do_direct_inference, ...
											  do_gene_identifiability, ...
											  forced_state_switch_pts);

                n_samples   = size(time_pred_probs, 1);
                n_correct_t = 0;
                n_correct_s = 0;
                for s = 1:n_samples
                    % Time prediction.
                    [max_prob, pred_label] = max(time_pred_probs(s, :));
                    if (pred_label == s)
                        n_correct_t = n_correct_t + 1;                        
                    end

                    time_pred_labels(((i_exp-1)*n_tries)+trial, s) = pred_label;

                    % Stage prediction.
                    [max_prob, pred_state_label] = max(state_pred_probs(s, :));
                        
                    %if (s <= 3)
		    if (s < forced_state_switch_pts(1))
		        if (pred_state_label == 1)
			    n_correct_s = n_correct_s + 1;
		        end
		    else
		        if (pred_state_label == 2)
			    n_correct_s = n_correct_s + 1;
		        end
		    end
		
		    stage_pred_labels(((i_exp-1)*n_tries)+trial, s) = pred_state_label;
                end

                acc_t_ng_eo_et(((i_exp-1)*n_tries)+trial) = n_correct_t / n_samples;
                acc_s_ng_eo_et(((i_exp-1)*n_tries)+trial) = n_correct_s / n_samples;
                
                %Update results.
                acc_results = [acc_t_ng_eo_et acc_s_ng_eo_et acc_t_bp_eo_et acc_s_bp_eo_et acc_t_mf_eo_et acc_s_mf_eo_et];    
                save(acc_results_file, 'acc_results', '-ASCII');

                save(time_labels_file, 'time_pred_labels', '-ASCII');
                save(stage_labels_file, 'stage_pred_labels', '-ASCII');

            end

        end

    end

    if ( strcmp(go_dag, 'bp') )

        % BP GO tags.
        %go_tags_file   = 'data/hoying_dataset/exp_one/go_tags_bp_w_anc_1282_subset';
        %clusters_file  = 'data/hoying_dataset/em_clustering_with_GO_tags/exp_one/bayesian_go_input_bp_w_anc_1282_subset.txt';
        %p_o_bar_c_file = 'data/hoying_dataset/em_clustering_with_GO_tags/exp_one/p_o_bar_c_bp_w_anc_1282_subset.txt';
        %model_mat_file = 'data/hoying_dataset/em_clustering_with_GO_tags/exp_one/model_bp_w_anc_1282_subset.mat';
        go_tags_file   = bp_go_tags_files{i_exp};
        clusters_file  = bp_clusters_files{i_exp};
        p_o_bar_c_file = bp_p_o_bar_c_files{i_exp};
        model_mat_file = bp_model_files{i_exp};

        for trial = 1:n_tries
            
            sum_pred_probs = sum(acc_from_file((((i_exp-1)*n_tries)+trial): ...
                                                (n_exps*n_tries), 3));
            
            if ( sum_pred_probs  > 0 )
            
                exp_str   = num2str(i_exp);
                trial_str = num2str(trial);

                disp(strcat('Skipping trial: ', trial_str, ' for experiment: ', exp_str));
                continue;

            else

                cluster_with_em_GO_tags_tied_params(train_file, ...
						    go_tags_file, ...
						    n_states, ...
						    n_clusters, ...
						    n_em_iters, ...
						    do_hard_cluster, ...
						    do_weight_GO_tags, ...
						    clusters_file, ...
						    p_o_bar_c_file, ...
						    model_mat_file, ...
						    forced_state_switch_pts);

                [time_pred_probs, state_pred_probs] = predict_max_lhood_label_tied_params(train_file, ...
										          test_file, ...
											  go_tags_file, ...
											  model_mat_file, ...
											  do_marginalize, ...
											  do_weight_GO_tags, ...
											  ignore_GO_tags_for_pred, ...
											  n_closest_genes, ...
											  do_weighted_expr_avg_for_pred, ...
											  do_direct_inference, ...
											  do_gene_identifiability, ...
											  forced_state_switch_pts);

                n_samples   = size(time_pred_probs, 1);
                n_correct_t = 0;
                n_correct_s = 0;
                for s = 1:n_samples
                    % Time prediction.
                    [max_prob, pred_label] = max(time_pred_probs(s, :));
                    if (pred_label == s)
                        n_correct_t = n_correct_t + 1;
                    end

                    time_pred_labels(((i_exp-1)*n_tries)+trial, max_n_samples+s) = pred_label;

                    % Stage prediction.
                    [max_prob, pred_state_label] = max(state_pred_probs(s, :));
                        
                    %if (s <= 3)
		    if (s < forced_state_switch_pts(1))
		        if (pred_state_label == 1)
			    n_correct_s = n_correct_s + 1;
		        end
		    else
		        if (pred_state_label == 2)
			    n_correct_s = n_correct_s + 1;
		        end
		    end
		
		    stage_pred_labels(((i_exp-1)*n_tries)+trial, s) = pred_state_label;
                end

                acc_t_bp_eo_et(((i_exp-1)*n_tries)+trial) = n_correct_t / n_samples;
                acc_s_bp_eo_et(((i_exp-1)*n_tries)+trial) = n_correct_s / n_samples;                                         
                
                %Update results.
                acc_results = [acc_t_ng_eo_et acc_s_ng_eo_et acc_t_bp_eo_et acc_s_bp_eo_et acc_t_mf_eo_et acc_s_mf_eo_et];    
                save(acc_results_file, 'acc_results', '-ASCII');

                save(time_labels_file, 'time_pred_labels', '-ASCII');
                save(stage_labels_file, 'stage_pred_labels', '-ASCII');

            end

        end

    end

    if ( strcmp(go_dag, 'mf') )

        % MF GO tags.
        %go_tags_file   = 'data/hoying_dataset/exp_one/go_tags_mf_w_anc_1282_subset';
        %clusters_file  = 'data/hoying_dataset/em_clustering_with_GO_tags/exp_one/bayesian_go_input_mf_w_anc_1282_subset.txt';
        %p_o_bar_c_file = 'data/hoying_dataset/em_clustering_with_GO_tags/exp_one/p_o_bar_c_mf_w_anc_1282_subset.txt';
        %model_mat_file = 'data/hoying_dataset/em_clustering_with_GO_tags/exp_one/model_mf_w_anc_1282_subset.mat';
        go_tags_file   = mf_go_tags_files{i_exp};
        clusters_file  = mf_clusters_files{i_exp};
        p_o_bar_c_file = mf_p_o_bar_c_files{i_exp};
        model_mat_file = mf_model_files{i_exp};

        for trial = 1:n_tries
            
            sum_pred_probs = sum(acc_from_file((((i_exp-1)*n_tries)+trial): ...
                                                (n_exps*n_tries), 5));
            
            if ( sum_pred_probs  > 0 )
            
                exp_str   = num2str(i_exp);
                trial_str = num2str(trial);

                disp(strcat('Skipping trial: ', trial_str, ' for experiment: ', exp_str));
                continue;

            else

                cluster_with_em_GO_tags_tied_params(train_file, ...
						    go_tags_file, ...
						    n_states, ...
						    n_clusters, ...
						    n_em_iters, ...
						    do_hard_cluster, ...
						    do_weight_GO_tags, ...
						    clusters_file, ...
						    p_o_bar_c_file, ...
						    model_mat_file, ...
						    forced_state_switch_pts);

                [time_pred_probs, state_pred_probs] = predict_max_lhood_label_tied_params(train_file, ...
										          test_file, ...
											  go_tags_file, ...
											  model_mat_file, ...
											  do_marginalize, ...
											  do_weight_GO_tags, ...
											  ignore_GO_tags_for_pred, ...
											  n_closest_genes, ...
											  do_weighted_expr_avg_for_pred, ...
											  do_direct_inference, ...
											  do_gene_identifiability, ...
											  forced_state_switch_pts);

                n_samples   = size(time_pred_probs, 1);
                n_correct_t = 0;
                n_correct_s = 0;
                for s = 1:n_samples
                    % Time prediction.
                    [max_prob, pred_label] = max(time_pred_probs(s, :));
                    if (pred_label == s)
                        n_correct_t = n_correct_t + 1;
                    end
                    
                    time_pred_labels(((i_exp-1)*n_tries)+trial, (2*max_n_samples)+s) = pred_label;

                    % Stage prediction.
                    [max_prob, pred_state_label] = max(state_pred_probs(s, :));
                        
                    %if (s <= 3)
		    if (s < forced_state_switch_pts(1))
		        if (pred_state_label == 1)
			    n_correct_s = n_correct_s + 1;
		        end
		    else
		        if (pred_state_label == 2)
			    n_correct_s = n_correct_s + 1;
		        end
		    end
		
		    stage_pred_labels(((i_exp-1)*n_tries)+trial, s) = pred_state_label;
                end

                acc_t_mf_eo_et(((i_exp-1)*n_tries)+trial) = n_correct_t / n_samples;
                acc_s_mf_eo_et(((i_exp-1)*n_tries)+trial) = n_correct_s / n_samples;
                
                %Update results.
                acc_results = [acc_t_ng_eo_et acc_s_ng_eo_et acc_t_bp_eo_et acc_s_bp_eo_et acc_t_mf_eo_et acc_s_mf_eo_et];    
                save(acc_results_file, 'acc_results', '-ASCII');

                save(time_labels_file, 'time_pred_labels', '-ASCII');
                save(stage_labels_file, 'stage_pred_labels', '-ASCII');

            end

        end

    end

end
