% Compute average accuracy of label prediction on held-out samples of Whitfield et al.
% data. 
%
% A number of models for joint clustering of gene expression levels and GO tags
% are learnt using EM with different random initializations. Accuracy of label
% prediction is computed for each model and the accuracies are averaged. Average
% accuracy of label prediction by clustering only gene expression levels is also
% computed.

function [] = test_predict_max_lhood_label_whitfield(n_tries, ...
                                                     acc_results_file, ...
                                                     go_dag, ...
                                                     n_closest_genes, ...
                                                     n_clusters, ...
                                                     stage_labels_file)

addpath(genpath('..'));

n_em_iters              = 40;
do_hard_cluster         = 1;
do_weight_GO_tags       = 0;
do_marginalize          = 1;
ignore_GO_tags_for_pred = 0;
do_weighted_expr_avg    = 0;
do_direct_inference     = 0;
do_gene_identifiability = 0;

results_dir = 'models/whitfield_dataset/em_clustering_with_GO_tags/pred-w-GO-tags-wo-weights';

train_files = {'data/whitfield_dataset/gene_expr_tt3_1099_cy_1.txt', ...
               'data/whitfield_dataset/gene_expr_tt3_1099_cy_1.txt', ...
               'data/whitfield_dataset/gene_expr_tt3_1099_cy_1.txt', ...
               'data/whitfield_dataset/gene_expr_tt3_1099_cy_2.txt', ...
               'data/whitfield_dataset/gene_expr_tt3_1099_cy_2.txt', ...
               'data/whitfield_dataset/gene_expr_tt3_1099_cy_2.txt', ...
               'data/whitfield_dataset/gene_expr_tt3_1099_cy_3.txt', ...
               'data/whitfield_dataset/gene_expr_tt3_1099_cy_3.txt', ...
               'data/whitfield_dataset/gene_expr_tt3_1099_cy_3.txt'};

test_files = {'data/whitfield_dataset/gene_expr_tt3_1099_cy_1.txt', ...
              'data/whitfield_dataset/gene_expr_tt3_1099_cy_2.txt', ...
              'data/whitfield_dataset/gene_expr_tt3_1099_cy_3.txt', ...
              'data/whitfield_dataset/gene_expr_tt3_1099_cy_2.txt', ...
              'data/whitfield_dataset/gene_expr_tt3_1099_cy_3.txt', ...
              'data/whitfield_dataset/gene_expr_tt3_1099_cy_1.txt', ...
              'data/whitfield_dataset/gene_expr_tt3_1099_cy_3.txt', ...
              'data/whitfield_dataset/gene_expr_tt3_1099_cy_1.txt', ...
              'data/whitfield_dataset/gene_expr_tt3_1099_cy_2.txt'};

ng_go_tags_files = {'data/whitfield_dataset/no_go_tags_1099', ...
                    'data/whitfield_dataset/no_go_tags_1099', ...
                    'data/whitfield_dataset/no_go_tags_1099', ...
                    'data/whitfield_dataset/no_go_tags_1099', ...
                    'data/whitfield_dataset/no_go_tags_1099', ...
                    'data/whitfield_dataset/no_go_tags_1099', ...
                    'data/whitfield_dataset/no_go_tags_1099', ...
                    'data/whitfield_dataset/no_go_tags_1099', ...
                    'data/whitfield_dataset/no_go_tags_1099'};

bp_go_tags_files = {'data/whitfield_dataset/go_tags_bp_w_anc_1099', ...
                    'data/whitfield_dataset/go_tags_bp_w_anc_1099', ...
                    'data/whitfield_dataset/go_tags_bp_w_anc_1099', ...
                    'data/whitfield_dataset/go_tags_bp_w_anc_1099', ...
                    'data/whitfield_dataset/go_tags_bp_w_anc_1099', ...
                    'data/whitfield_dataset/go_tags_bp_w_anc_1099', ...
                    'data/whitfield_dataset/go_tags_bp_w_anc_1099', ...
                    'data/whitfield_dataset/go_tags_bp_w_anc_1099', ...
                    'data/whitfield_dataset/go_tags_bp_w_anc_1099'};
                    
mf_go_tags_files = {'data/whitfield_dataset/go_tags_mf_w_anc_1099', ...
                    'data/whitfield_dataset/go_tags_mf_w_anc_1099', ...
                    'data/whitfield_dataset/go_tags_mf_w_anc_1099', ...
                    'data/whitfield_dataset/go_tags_mf_w_anc_1099', ...
                    'data/whitfield_dataset/go_tags_mf_w_anc_1099', ...
                    'data/whitfield_dataset/go_tags_mf_w_anc_1099', ...
                    'data/whitfield_dataset/go_tags_mf_w_anc_1099', ...
                    'data/whitfield_dataset/go_tags_mf_w_anc_1099', ...
                    'data/whitfield_dataset/go_tags_mf_w_anc_1099'};
                    
ng_clusters_files = {strcat(results_dir, '/', 'bayesian_go_input_ng_1099_cy_1'), ...
                     strcat(results_dir, '/', 'bayesian_go_input_ng_1099_cy_1'), ...
                     strcat(results_dir, '/', 'bayesian_go_input_ng_1099_cy_1'), ...
                     strcat(results_dir, '/', 'bayesian_go_input_ng_1099_cy_2'), ...
                     strcat(results_dir, '/', 'bayesian_go_input_ng_1099_cy_2'), ...
                     strcat(results_dir, '/', 'bayesian_go_input_ng_1099_cy_2'), ...
                     strcat(results_dir, '/', 'bayesian_go_input_ng_1099_cy_3'), ...
                     strcat(results_dir, '/', 'bayesian_go_input_ng_1099_cy_3'), ...
                     strcat(results_dir, '/', 'bayesian_go_input_ng_1099_cy_3')};

bp_clusters_files = {strcat(results_dir, '/', 'bayesian_go_input_bp_1099_cy_1'), ...
                     strcat(results_dir, '/', 'bayesian_go_input_bp_1099_cy_1'), ...
                     strcat(results_dir, '/', 'bayesian_go_input_bp_1099_cy_1'), ...
                     strcat(results_dir, '/', 'bayesian_go_input_bp_1099_cy_2'), ...
                     strcat(results_dir, '/', 'bayesian_go_input_bp_1099_cy_2'), ...
                     strcat(results_dir, '/', 'bayesian_go_input_bp_1099_cy_2'), ...
                     strcat(results_dir, '/', 'bayesian_go_input_bp_1099_cy_3'), ...
                     strcat(results_dir, '/', 'bayesian_go_input_bp_1099_cy_3'), ...
                     strcat(results_dir, '/', 'bayesian_go_input_bp_1099_cy_3')};
                 
mf_clusters_files = {strcat(results_dir, '/', 'bayesian_go_input_mf_1099_cy_1'), ...
                     strcat(results_dir, '/', 'bayesian_go_input_mf_1099_cy_1'), ...
                     strcat(results_dir, '/', 'bayesian_go_input_mf_1099_cy_1'), ...
                     strcat(results_dir, '/', 'bayesian_go_input_mf_1099_cy_2'), ...
                     strcat(results_dir, '/', 'bayesian_go_input_mf_1099_cy_2'), ...
                     strcat(results_dir, '/', 'bayesian_go_input_mf_1099_cy_2'), ...
                     strcat(results_dir, '/', 'bayesian_go_input_mf_1099_cy_3'), ...
                     strcat(results_dir, '/', 'bayesian_go_input_mf_1099_cy_3'), ...
                     strcat(results_dir, '/', 'bayesian_go_input_mf_1099_cy_3')};

ng_p_o_bar_c_files ={strcat(results_dir, '/', 'p_o_bar_c_ng_1099_cy_1'), ...
                     strcat(results_dir, '/', 'p_o_bar_c_ng_1099_cy_1'), ...
                     strcat(results_dir, '/', 'p_o_bar_c_ng_1099_cy_1'), ...
                     strcat(results_dir, '/', 'p_o_bar_c_ng_1099_cy_2'), ...
                     strcat(results_dir, '/', 'p_o_bar_c_ng_1099_cy_2'), ...
                     strcat(results_dir, '/', 'p_o_bar_c_ng_1099_cy_2'), ...
                     strcat(results_dir, '/', 'p_o_bar_c_ng_1099_cy_3'), ...
                     strcat(results_dir, '/', 'p_o_bar_c_ng_1099_cy_3'), ...
                     strcat(results_dir, '/', 'p_o_bar_c_ng_1099_cy_3')};

bp_p_o_bar_c_files ={strcat(results_dir, '/', 'p_o_bar_c_bp_w_anc_1099_cy_1'), ...
                     strcat(results_dir, '/', 'p_o_bar_c_bp_w_anc_1099_cy_1'), ...
                     strcat(results_dir, '/', 'p_o_bar_c_bp_w_anc_1099_cy_1'), ...
                     strcat(results_dir, '/', 'p_o_bar_c_bp_w_anc_1099_cy_2'), ...
                     strcat(results_dir, '/', 'p_o_bar_c_bp_w_anc_1099_cy_2'), ...
                     strcat(results_dir, '/', 'p_o_bar_c_bp_w_anc_1099_cy_2'), ...
                     strcat(results_dir, '/', 'p_o_bar_c_bp_w_anc_1099_cy_3'), ...
                     strcat(results_dir, '/', 'p_o_bar_c_bp_w_anc_1099_cy_3'), ...
                     strcat(results_dir, '/', 'p_o_bar_c_bp_w_anc_1099_cy_3')};

mf_p_o_bar_c_files ={strcat(results_dir, '/', 'p_o_bar_c_mf_w_anc_1099_cy_1'), ...
                     strcat(results_dir, '/', 'p_o_bar_c_mf_w_anc_1099_cy_1'), ...
                     strcat(results_dir, '/', 'p_o_bar_c_mf_w_anc_1099_cy_1'), ...
                     strcat(results_dir, '/', 'p_o_bar_c_mf_w_anc_1099_cy_2'), ...
                     strcat(results_dir, '/', 'p_o_bar_c_mf_w_anc_1099_cy_2'), ...
                     strcat(results_dir, '/', 'p_o_bar_c_mf_w_anc_1099_cy_2'), ...
                     strcat(results_dir, '/', 'p_o_bar_c_mf_w_anc_1099_cy_3'), ...
                     strcat(results_dir, '/', 'p_o_bar_c_mf_w_anc_1099_cy_3'), ...
                     strcat(results_dir, '/', 'p_o_bar_c_mf_w_anc_1099_cy_3')};

ng_model_files = {strcat(results_dir, '/', 'model_ng_1099_cy_1.mat'), ... 
                  strcat(results_dir, '/', 'model_ng_1099_cy_1.mat'), ...
                  strcat(results_dir, '/', 'model_ng_1099_cy_1.mat'), ...
                  strcat(results_dir, '/', 'model_ng_1099_cy_2.mat'), ...
                  strcat(results_dir, '/', 'model_ng_1099_cy_2.mat'), ...
                  strcat(results_dir, '/', 'model_ng_1099_cy_2.mat'), ...
                  strcat(results_dir, '/', 'model_ng_1099_cy_3.mat'), ...
                  strcat(results_dir, '/', 'model_ng_1099_cy_3.mat'), ...
                  strcat(results_dir, '/', 'model_ng_1099_cy_3.mat')};

bp_model_files = {strcat(results_dir, '/', 'model_bp_w_anc_1099_cy_1.mat'), ... 
                  strcat(results_dir, '/', 'model_bp_w_anc_1099_cy_1.mat'), ...
                  strcat(results_dir, '/', 'model_bp_w_anc_1099_cy_1.mat'), ...
                  strcat(results_dir, '/', 'model_bp_w_anc_1099_cy_2.mat'), ...
                  strcat(results_dir, '/', 'model_bp_w_anc_1099_cy_2.mat'), ...
                  strcat(results_dir, '/', 'model_bp_w_anc_1099_cy_2.mat'), ...
                  strcat(results_dir, '/', 'model_bp_w_anc_1099_cy_3.mat'), ...
                  strcat(results_dir, '/', 'model_bp_w_anc_1099_cy_3.mat'), ...
                  strcat(results_dir, '/', 'model_bp_w_anc_1099_cy_3.mat')};

mf_model_files = {strcat(results_dir, '/', 'model_mf_w_anc_1099_cy_1.mat'), ... 
                  strcat(results_dir, '/', 'model_mf_w_anc_1099_cy_1.mat'), ...
                  strcat(results_dir, '/', 'model_mf_w_anc_1099_cy_1.mat'), ...
                  strcat(results_dir, '/', 'model_mf_w_anc_1099_cy_2.mat'), ...
                  strcat(results_dir, '/', 'model_mf_w_anc_1099_cy_2.mat'), ...
                  strcat(results_dir, '/', 'model_mf_w_anc_1099_cy_2.mat'), ...
                  strcat(results_dir, '/', 'model_mf_w_anc_1099_cy_3.mat'), ...
                  strcat(results_dir, '/', 'model_mf_w_anc_1099_cy_3.mat'), ...
                  strcat(results_dir, '/', 'model_mf_w_anc_1099_cy_3.mat')};

n_exps = length(train_files);

acc_s_ng = zeros(n_exps*n_tries, 1);
acc_s_bp = zeros(n_exps*n_tries, 1);
acc_s_mf = zeros(n_exps*n_tries, 1);

% Making script restartable.
if (exist(acc_results_file , 'file'))
    
    acc_from_file  = load(acc_results_file);
    sum_pred_probs = sum(acc_from_file(:));    

    if (sum_pred_probs > 0)
        acc_s_ng = acc_from_file(:, 1);
        acc_s_bp = acc_from_file(:, 2);
        acc_s_mf = acc_from_file(:, 3);
    end

else
    
    acc_from_file = [acc_s_ng acc_s_bp acc_s_mf];

end

% Writing stage prediction labels.
max_n_samples     = 17;
stage_pred_labels = zeros(n_exps*n_tries, 3*max_n_samples);

for i_exp = 1:n_exps
    train_file = train_files{i_exp};
    test_file  = test_files{i_exp};

    if ( strcmp(go_dag, 'ng') )

        % No GO tags.
        go_tags_file   = ng_go_tags_files{i_exp};
        clusters_file  = ng_clusters_files{i_exp};
        p_o_bar_c_file = ng_p_o_bar_c_files{i_exp};
        model_mat_file = ng_model_files{i_exp};

        for trial = 1:n_tries
            
            sum_pred_probs = sum(acc_from_file((((i_exp-1)*n_tries)+trial): ...
                                                    (n_exps*n_tries), 1));
                
            if ( sum_pred_probs  > 0 )
                
                exp_str   = num2str(i_exp);
                trial_str = num2str(trial);

                disp(strcat('Skipping trial: ', trial_str, ' for experiment: ', exp_str));
                continue;

            else
            
                cluster_with_em_GO_tags(train_file, ...
                                        go_tags_file, ...
                                        n_clusters, ...
                                        n_em_iters, ...
                                        do_hard_cluster, ...
                                        do_weight_GO_tags, ...
                                        clusters_file, ...
                                        p_o_bar_c_file, ...
                                        model_mat_file);

                pred_probs = predict_max_lhood_label(train_file, ...
                                                     test_file, ...
                                                     go_tags_file, ...
                                                     model_mat_file, ...
                                                     do_marginalize, ...
                                                     do_weight_GO_tags, ...
                                                     ignore_GO_tags_for_pred, ...
                                                     n_closest_genes, ...
                                                     do_weighted_expr_avg, ...
                                                     do_direct_inference, ...
                                                     do_gene_identifiability);
                
                n_samples   = size(pred_probs, 1);
                n_correct_s = 0;

                for s = 1:n_samples
                    
                    if (~isempty(strfind(train_file, 'cy_1')))

                        stage_probs = [sum(pred_probs(s, 1:4)), ...
                                       sum(pred_probs(s, 5:14))];

                    elseif (~isempty(strfind(train_file, 'cy_2')))

                        stage_probs = [sum(pred_probs(s, 1:5)), ...
                                       sum(pred_probs(s, 6:14))];

                    else

                        stage_probs = [sum(pred_probs(s, 1:5)), ...
                                       sum(pred_probs(s, 6:17))];
                        
                    end
                        
                    [max_prob, pred_stage] = max(stage_probs);

                    stage_pred_labels(((i_exp-1)*n_tries)+trial, s) = pred_stage;

                    if (~isempty(strfind(test_file, 'cy_1')))

                        if ( (s >= 1) & (s <= 4) )                
                            if (pred_stage == 1)
                                n_correct_s = n_correct_s + 1;
                            end
                        else %( (s >= 4) & (s <= 14) )
                            if (pred_stage == 2)
                                n_correct_s = n_correct_s + 1;
                            end
                        end

                    elseif (~isempty(strfind(test_file, 'cy_2')))

                        if ( (s >= 1) & (s <= 5) )                
                            if (pred_stage == 1)
                                n_correct_s = n_correct_s + 1;
                            end
                        else %( (s >= 6) & (s <= 14) )
                            if (pred_stage == 2)
                                n_correct_s = n_correct_s + 1;
                            end
                        end
                    
                    else

                        if ( (s >= 1) & (s <= 5) )                
                            if (pred_stage == 1)
                                n_correct_s = n_correct_s + 1;
                            end
                        else %( (s >= 6) & (s <= 17) )
                            if (pred_stage == 2)
                                n_correct_s = n_correct_s + 1;
                            end
                        end

                    end
                end
                acc_s_ng(((i_exp-1)*n_tries)+trial) = n_correct_s / n_samples;        
                
                %Update results.
                acc_results = [acc_s_ng acc_s_bp acc_s_mf];    
                save(acc_results_file, 'acc_results', '-ASCII');

                save(stage_labels_file, 'stage_pred_labels', '-ASCII');
            end

        end

    end

    if ( strcmp(go_dag, 'bp') )

        % BP GO tags.
        go_tags_file   = bp_go_tags_files{i_exp};
        clusters_file  = bp_clusters_files{i_exp};
        p_o_bar_c_file = bp_p_o_bar_c_files{i_exp};
        model_mat_file = bp_model_files{i_exp};

        for trial = 1:n_tries
            
            sum_pred_probs = sum(acc_from_file((((i_exp-1)*n_tries)+trial): ...
                                                    (n_exps*n_tries), 2));
                
            if ( sum_pred_probs  > 0 )
                
                exp_str   = num2str(i_exp);
                trial_str = num2str(trial);

                disp(strcat('Skipping trial: ', trial_str, ' for experiment: ', exp_str));
                continue;

            else
            
                cluster_with_em_GO_tags(train_file, ...
                                        go_tags_file, ...
                                        n_clusters, ...
                                        n_em_iters, ...
                                        do_hard_cluster, ...
                                        do_weight_GO_tags, ...
                                        clusters_file, ...
                                        p_o_bar_c_file, ...
                                        model_mat_file);

                pred_probs = predict_max_lhood_label(train_file, ...
                                                     test_file, ...
                                                     go_tags_file, ...
                                                     model_mat_file, ...
                                                     do_marginalize, ...
                                                     do_weight_GO_tags, ...
                                                     ignore_GO_tags_for_pred, ...
                                                     n_closest_genes, ...
                                                     do_weighted_expr_avg, ...
                                                     do_direct_inference, ...
                                                     do_gene_identifiability);
                
                n_samples   = size(pred_probs, 1);
                n_correct_s = 0;

                for s = 1:n_samples
                
                    if (~isempty(strfind(train_file, 'cy_1')))

                        stage_probs = [sum(pred_probs(s, 1:4)), ...
                                       sum(pred_probs(s, 5:14))];

                    elseif (~isempty(strfind(train_file, 'cy_2')))

                        stage_probs = [sum(pred_probs(s, 1:5)), ...
                                       sum(pred_probs(s, 6:14))];

                    else

                        stage_probs = [sum(pred_probs(s, 1:5)), ...
                                       sum(pred_probs(s, 6:17))];
                        
                    end
                        
                    [max_prob, pred_stage] = max(stage_probs);

                    stage_pred_labels(((i_exp-1)*n_tries)+trial, max_n_samples+s) = pred_stage;

                    if (~isempty(strfind(test_file, 'cy_1')))

                        if ( (s >= 1) & (s <= 4) )                
                            if (pred_stage == 1)
                                n_correct_s = n_correct_s + 1;
                            end
                        else %( (s >= 4) & (s <= 14) )
                            if (pred_stage == 2)
                                n_correct_s = n_correct_s + 1;
                            end
                        end

                    elseif (~isempty(strfind(test_file, 'cy_2')))

                        if ( (s >= 1) & (s <= 5) )                
                            if (pred_stage == 1)
                                n_correct_s = n_correct_s + 1;
                            end
                        else %( (s >= 6) & (s <= 14) )
                            if (pred_stage == 2)
                                n_correct_s = n_correct_s + 1;
                            end
                        end
                    
                    else

                        if ( (s >= 1) & (s <= 5) )                
                            if (pred_stage == 1)
                                n_correct_s = n_correct_s + 1;
                            end
                        else %( (s >= 6) & (s <= 17) )
                            if (pred_stage == 2)
                                n_correct_s = n_correct_s + 1;
                            end
                        end

                    end
                end
                acc_s_bp(((i_exp-1)*n_tries)+trial) = n_correct_s / n_samples;

                %Update results.
                acc_results = [acc_s_ng acc_s_bp acc_s_mf];    
                save(acc_results_file, 'acc_results', '-ASCII');

                save(stage_labels_file, 'stage_pred_labels', '-ASCII');
            end

        end

    end

    if ( strcmp(go_dag, 'mf') )

        % MF GO tags.
        go_tags_file   = mf_go_tags_files{i_exp};
        clusters_file  = mf_clusters_files{i_exp};
        p_o_bar_c_file = mf_p_o_bar_c_files{i_exp};
        model_mat_file = mf_model_files{i_exp};

        for trial = 1:n_tries
            
            sum_pred_probs = sum(acc_from_file((((i_exp-1)*n_tries)+trial): ...
                                                    (n_exps*n_tries), 3));
                
            if ( sum_pred_probs  > 0 )
                
                exp_str   = num2str(i_exp);
                trial_str = num2str(trial);

                disp(strcat('Skipping trial: ', trial_str, ' for experiment: ', exp_str));
                continue;

            else

                cluster_with_em_GO_tags(train_file, ...
                                        go_tags_file, ...
                                        n_clusters, ...
                                        n_em_iters, ...
                                        do_hard_cluster, ...
                                        do_weight_GO_tags, ...
                                        clusters_file, ...
                                        p_o_bar_c_file, ...
                                        model_mat_file);

                pred_probs = predict_max_lhood_label(train_file, ...
                                                     test_file, ...
                                                     go_tags_file, ...
                                                     model_mat_file, ...
                                                     do_marginalize, ...
                                                     do_weight_GO_tags, ...
                                                     ignore_GO_tags_for_pred, ...
                                                     n_closest_genes, ...
                                                     do_weighted_expr_avg, ...
                                                     do_direct_inference, ...
                                                     do_gene_identifiability);
                
                n_samples   = size(pred_probs, 1);
                n_correct_s = 0;

                for s = 1:n_samples
                    
                    if (~isempty(strfind(train_file, 'cy_1')))

                        stage_probs = [sum(pred_probs(s, 1:4)), ...
                                       sum(pred_probs(s, 5:14))];

                    elseif (~isempty(strfind(train_file, 'cy_2')))

                        stage_probs = [sum(pred_probs(s, 1:5)), ...
                                       sum(pred_probs(s, 6:14))];

                    else

                        stage_probs = [sum(pred_probs(s, 1:5)), ...
                                       sum(pred_probs(s, 6:17))];
                        
                    end
                        
                    [max_prob, pred_stage] = max(stage_probs);

                    stage_pred_labels(((i_exp-1)*n_tries)+trial, (2*max_n_samples)+s) = pred_stage;

                    if (~isempty(strfind(test_file, 'cy_1')))

                        if ( (s >= 1) & (s <= 4) )                
                            if (pred_stage == 1)
                                n_correct_s = n_correct_s + 1;
                            end
                        else %( (s >= 4) & (s <= 14) )
                            if (pred_stage == 2)
                                n_correct_s = n_correct_s + 1;
                            end
                        end

                    elseif (~isempty(strfind(test_file, 'cy_2')))

                        if ( (s >= 1) & (s <= 5) )                
                            if (pred_stage == 1)
                                n_correct_s = n_correct_s + 1;
                            end
                        else %( (s >= 6) & (s <= 14) )
                            if (pred_stage == 2)
                                n_correct_s = n_correct_s + 1;
                            end
                        end
                    
                    else

                        if ( (s >= 1) & (s <= 5) )                
                            if (pred_stage == 1)
                                n_correct_s = n_correct_s + 1;
                            end
                        else %( (s >= 6) & (s <= 17) )
                            if (pred_stage == 2)
                                n_correct_s = n_correct_s + 1;
                            end
                        end

                    end

                end
                acc_s_mf(((i_exp-1)*n_tries)+trial) = n_correct_s / n_samples;
                
                %Update results.
                acc_results = [acc_s_ng acc_s_bp acc_s_mf];    
                save(acc_results_file, 'acc_results', '-ASCII');

                save(stage_labels_file, 'stage_pred_labels', '-ASCII');
            end

        end

    end

end
